--------------------------------------------------------------------------------
-- Locals

-- Comm stuff
local PRE = "TypI"
local STOP = "stop"
local START_PARTY = "start-party"
local START_TELL = "start-tell"

-- Strings
local PARTY_FORMAT, TELL_FORMAT = nil, nil

local playerName = UnitName("player")
local lastTarget = nil
local totalElapsed = 0
local updateFrame = CreateFrame("Frame")

--------------------------------------------------------------------------------
-- Unit name coloring

local guildMemberList = nil
local hexColors = {
	WTF = "|cffa0a0a0"
}
for k, v in pairs(RAID_CLASS_COLORS) do
	hexColors[k] = "|cff" .. string.format("%02x%02x%02x", v.r * 255, v.g * 255, v.b * 255)
end
local coloredName = setmetatable({}, {__index =
	function(self, key)
		if type(key) == "nil" then return nil end
		local class = guildMemberList and guildMemberList[key] or select(2, UnitClass(key)) or "WTF"
		self[key] = hexColors[class]  .. key .. "|r"
		return self[key]
	end
})

--------------------------------------------------------------------------------
-- Set up the display

local addon = CreateFrame("Button", "TypingIndicator", UIParent)

local function dragStart()
	this:StartMoving()
end
local function dragStop()
	this:StopMovingOrSizing()
end

addon:SetClampedToScreen(true)
addon:SetHeight(14)
addon:SetWidth(10)
addon:SetFrameStrata("LOW")
addon:SetToplevel(1)
addon:SetPoint("TOPLEFT", UIParent, "TOPLEFT", 150, -150)
addon:SetBackdropBorderColor(1, 1, 1, 0)
addon:SetBackdropColor(1, 1, 1, 0)
addon:SetMovable(true)
addon:RegisterForDrag("LeftButton")
addon:SetScript("OnDragStart", dragStart)
addon:SetScript("OnDragStop", dragStop)
addon:SetScript("OnHide", dragStop)

local indicator = addon:CreateFontString(nil, "OVERLAY")
indicator:SetFont(GameFontNormal:GetFont(), 14)
indicator:SetVertexColor(0, 1, 0)
indicator:SetHeight(14)
indicator:SetText("")
indicator:ClearAllPoints()
indicator:SetAllPoints(addon)

local function setText(t)
	indicator:SetText(t)
	addon:SetWidth(indicator:GetStringWidth())
	if not t or t:len() == 0 then
		addon:Hide()
	else
		addon:Show()
	end
end

--------------------------------------------------------------------------------
-- Chat command

_G.SlashCmdList["TYPI_SHORTHAND"] = function()
	local t = indicator:GetText()
	if not t or t:len() == 0 then
		setText(TELL_FORMAT:format(coloredName[playerName]))
		updateFrame:Hide()
	else
		setText("")
		updateFrame:Show()
	end
end
_G.SLASH_TYPI_SHORTHAND1 = "/typi"

--------------------------------------------------------------------------------
-- Event and comm handling

addon:SetScript("OnEvent", function(self, event, ...)
	self[event](self, ...)
end)
addon:RegisterEvent("CHAT_MSG_ADDON")
addon:RegisterEvent("GUILD_ROSTER_UPDATE")
addon:RegisterEvent("PLAYER_LOGIN")
function addon:PLAYER_LOGIN()
	-- ChatTypeInfo doesn't seem to be populated with the right colors until PLAYER_LOGIN
	local cW = ChatTypeInfo.WHISPER
	local cP = ChatTypeInfo.PARTY
	local tellColor = "|cff" .. string.format("%02x%02x%02x", cW.r * 255, cW.g * 255, cW.b * 255)
	local partyColor = "|cff" .. string.format("%02x%02x%02x", cP.r * 255, cP.g * 255, cP.b * 255)
	PARTY_FORMAT = "%s " .. partyColor .. "is partying ...|r"
	TELL_FORMAT = "%s " .. tellColor .. "is whispering ...|r"
end
function addon:CHAT_MSG_ADDON(pre, msg, dist, sender)
	if not msg or not sender or pre ~= PRE then return end
	if msg == STOP then
		setText("")
	elseif msg == START_PARTY then
		--if playerName == sender then return end
		setText(PARTY_FORMAT:format(coloredName[sender]))
	elseif msg == START_TELL then
		setText(TELL_FORMAT:format(coloredName[sender]))
	end
end
function addon:GUILD_ROSTER_UPDATE()
	if not guildMemberList then
		self:UnregisterEvent("GUILD_ROSTER_UPDATE")
		for k in pairs(coloredName) do coloredName[k] = nil end
		guildMemberList = {}
		local offline = GetGuildRosterShowOffline()
		local selection = GetGuildRosterSelection()
		SetGuildRosterShowOffline(true)
		SetGuildRosterSelection(0)
		GetGuildRosterInfo(0)
		local numGuildMembers = GetNumGuildMembers()
		for i = 1, numGuildMembers do
			local name, _, _, _, _, _, _, _, _, _, class = GetGuildRosterInfo(i)
			guildMemberList[name] = class
		end
		SetGuildRosterShowOffline(offline)
		SetGuildRosterSelection(selection)
	end
end

--------------------------------------------------------------------------------
-- Track who we're whispering

-- Sends a message to the last tell target to clear the typing indicator
local function clearIndicator()
	if not lastTarget then return end
	if lastTarget == "PARTY" then
		SendAddonMessage(PRE, STOP, "PARTY")
	else
		SendAddonMessage(PRE, STOP, "WHISPER", lastTarget)
	end
	lastTarget = nil
	totalElapsed = 0
end

-- Sends a message to the current tell target to
local function startIndicator(target)
	lastTarget = target
	if target == "PARTY" then
		SendAddonMessage(PRE, START_PARTY, "PARTY")
	else
		SendAddonMessage(PRE, START_TELL, "WHISPER", target)
	end
end

-- OnUpdate function to handle a typing timeout, if nothing is typed for 5
-- seconds, we send a typing indicator clear to the last tell target.
updateFrame:SetScript("OnUpdate", function(self, elapsed)
	if not lastTarget then return end
	totalElapsed = totalElapsed + elapsed
	if totalElapsed > 5 then
		clearIndicator()
	end
end)
updateFrame:Show()

hooksecurefunc("ChatEdit_OnTextChanged", function()
	if not this then return end
	local t = this:GetAttribute("chatType")
	if not t then return end
	local target = nil
	if t == "WHISPER" then
		target = this:GetAttribute("tellTarget")
	elseif t == "PARTY" then
		target = t
	else
		return
	end
	if not target or (lastTarget and target ~= lastTarget) then
		clearIndicator()
	end
	if target and not lastTarget then
		local t = this:GetText()
		if not t or t:trim():len() < 1 or t:find("^/") then return end
		startIndicator(target)
	end
	totalElapsed = 0
end)

local onHide = ChatFrameEditBox:GetScript("OnHide")
ChatFrameEditBox:SetScript("OnHide", function(...)
	clearIndicator()
	if onHide then onHide(...) end
end)

